@@ -25,7 +25,7 @@ WS_CURRENT_APPS = "apps/status"
 REST_BASE_URL = "http://{0}:{1}/as/{2}"
 REST_CHANNEL_LIST = "services"
 REST_RECORDING_DETAILS = "pvr/details/{0}"
-REST_PATH_INFO = "system/information"
+REST_PATH_SYSTEMINFO = "system/information"
 REST_PATH_DEVICEINFO = "system/deviceinformation"
 
 # Sky specific constants
