@@ -29,7 +29,7 @@ from .const import (
     REST_BASE_URL,
     REST_CHANNEL_LIST,
     REST_RECORDING_DETAILS,
-    REST_PATH_INFO,
+    REST_PATH_SYSTEMINFO,
     REST_PATH_DEVICEINFO,
     CURRENT_URI,
     CURRENT_TRANSPORT_STATE,
@@ -52,6 +52,7 @@ from .const import TEST_CHANNEL_LIST
 from .channel import Channel
 from .programme import RecordedProgramme
 from .media import Media
+from .device import Device
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -65,10 +66,8 @@ class SkyQRemote:
         self, host, overrideCountry=None, test_channel=None, port=49160, jsonport=9006,
     ):
         """Stand up a new SkyQ box."""
-        self.host = host
-        self.country = None
-        self.serialNumber = None
-        self.deviceSetup = False
+        self._host = host
+        self._serialNumber = None
         self._channel = None
         self._test_channel = test_channel
         self._port = port
@@ -77,8 +76,9 @@ class SkyQRemote:
         self._soapControlURL = None
         self._lastEpg = None
         self._currentApp = APP_EPG
+        self.deviceSetup = False
 
-        self.powerStatus()
+        self._setupDevice()
 
     def powerStatus(self) -> str:
         """Get the power status of the Sky Q box."""
@@ -88,7 +88,7 @@ class SkyQRemote:
         if self._soapControlURL is None:
             return SKY_STATE_OFF
         try:
-            output = self._http_json(REST_PATH_INFO)
+            output = self._retrieveSystemInformation()
             if "activeStandby" in output and output["activeStandby"] is False:
                 return SKY_STATE_ON
             return SKY_STATE_STANDBY
@@ -99,11 +99,11 @@ class SkyQRemote:
             return SKY_STATE_STANDBY
         except (requests.exceptions.ConnectionError):
             _LOGGER.info(
-                f"I0010 - Device has control URL but connection request failed: {self.host}"
+                f"I0010 - Device has control URL but connection request failed: {self._host}"
             )
             return SKY_STATE_OFF
         except Exception as err:
-            _LOGGER.exception(f"X0060 - Error occurred: {self.host} : {err}")
+            _LOGGER.exception(f"X0060 - Error occurred: {self._host} : {err}")
             return SKY_STATE_STANDBY
 
     def getCurrentState(self):
@@ -257,7 +257,7 @@ class SkyQRemote:
 
             return programme
         except Exception as err:
-            _LOGGER.exception(f"X0030 - Error occurred: {self.host} : {sid} : {err}")
+            _LOGGER.exception(f"X0030 - Error occurred: {self._host} : {sid} : {err}")
             return None
 
     def getRecordingJSON(self, pvrId):
@@ -312,24 +312,75 @@ class SkyQRemote:
 
         return programme
 
+    def getDeviceInformationJSON(self):
+        """Get the device information from the SkyQ box as json."""
+        device = self.getDeviceInformation()
+        if not device:
+            return None
+
+        return device.as_json()
+
+    def getDeviceInformation(self):
+        """Get the device information from the SkyQ box."""
+        device = None
+        deviceInfo = self._retrieveDeviceInformation()
+
+        if deviceInfo:
+            systemInfo = self._retrieveSystemInformation()
+            ASVersion = deviceInfo["ASVersion"]
+            IPAddress = deviceInfo["IPAddress"]
+            countryCode = deviceInfo["countryCode"]
+            hardwareModel = systemInfo["hardwareModel"]
+            hardwareName = deviceInfo["hardwareName"]
+            manufacturer = systemInfo["manufacturer"]
+            modelNumber = deviceInfo["modelNumber"]
+            serialNumber = deviceInfo["serialNumber"]
+            versionNumber = deviceInfo["versionNumber"]
+
+            epgCountryCode = None
+            if self._overrideCountry:
+                epgCountryCode = self._overrideCountry
+            else:
+                epgCountryCode = countryCode.upper()
+            if not epgCountryCode:
+                _LOGGER.error(f"E0050 - No country identified: {self._host}")
+                return None
+
+            if epgCountryCode in KNOWN_COUNTRIES:
+                epgCountryCode = KNOWN_COUNTRIES[epgCountryCode]
+
+            device = Device(
+                ASVersion,
+                IPAddress,
+                countryCode,
+                epgCountryCode,
+                hardwareModel,
+                hardwareName,
+                manufacturer,
+                modelNumber,
+                serialNumber,
+                versionNumber,
+            )
+        return device
+
     def press(self, sequence):
         """Issue the specified sequence of commands to SkyQ box."""
         if isinstance(sequence, list):
             for item in sequence:
                 if item.casefold() not in self.commands:
-                    _LOGGER.error(f"E0010 - Invalid command: {self.host} : {item}")
+                    _LOGGER.error(f"E0010 - Invalid command: {self._host} : {item}")
                     break
                 self._sendCommand(self.commands[item.casefold()])
                 time.sleep(0.5)
         else:
             if sequence not in self.commands:
-                _LOGGER.error(f"E0020 - Invalid command: {self.host} : {sequence}")
+                _LOGGER.error(f"E0020 - Invalid command: {self._host} : {sequence}")
             else:
                 self._sendCommand(self.commands[sequence.casefold()])
 
     def _http_json(self, path, headers=None) -> str:
         response = requests.get(
-            REST_BASE_URL.format(self.host, self._jsonport, path),
+            REST_BASE_URL.format(self._host, self._jsonport, path),
             timeout=TIMEOUT,
             headers=headers,
         )
@@ -338,7 +389,7 @@ class SkyQRemote:
     def _getSoapControlURL(self, descriptionIndex):
         try:
             descriptionUrl = SOAP_DESCRIPTION_BASE_URL.format(
-                self.host, descriptionIndex
+                self._host, descriptionIndex
             )
             headers = {"User-Agent": SOAP_USER_AGENT}
             resp = requests.get(descriptionUrl, headers=headers, timeout=TIMEOUT)
@@ -358,21 +409,21 @@ class SkyQRemote:
                     return {"url": None, "status": "Not Found"}
                 return {
                     "url": SOAP_CONTROL_BASE_URL.format(
-                        self.host, playService["controlURL"]
+                        self._host, playService["controlURL"]
                     ),
                     "status": "OK",
                 }
             return {"url": None, "status": "Not Found"}
         except (requests.exceptions.Timeout):
             _LOGGER.warning(
-                f"W0010 - Control URL not accessible: {self.host} : {descriptionUrl}"
+                f"W0010 - Control URL not accessible: {self._host} : {descriptionUrl}"
             )
             return {"url": None, "status": "Error"}
         except (requests.exceptions.ConnectionError) as err:
-            _LOGGER.exception(f"X0070 - Connection error: {self.host} : {err}")
+            _LOGGER.exception(f"X0070 - Connection error: {self._host} : {err}")
             return {"url": None, "status": "Error"}
         except Exception as err:
-            _LOGGER.exception(f"X0010 - Other error occurred: {self.host} : {err}")
+            _LOGGER.exception(f"X0010 - Other error occurred: {self._host} : {err}")
             return {"url": None, "status": "Error"}
 
     def _callSkySOAPService(self, method):
@@ -400,17 +451,17 @@ class SkyQRemote:
 
     def _callSkyWebSocket(self, method):
         try:
-            ws = websocket.create_connection(WS_BASE_URL.format(self.host, method))
+            ws = websocket.create_connection(WS_BASE_URL.format(self._host, method))
             response = json.loads(ws.recv())
             ws.close()
             return response
         except (TimeoutError) as err:
             _LOGGER.warning(
-                f"W0020 - Websocket call failed: {self.host} : {method} : {err}"
+                f"W0020 - Websocket call failed: {self._host} : {method} : {err}"
             )
             return {"url": None, "status": "Error"}
         except Exception as err:
-            _LOGGER.exception(f"X0020 - Error occurred: {self.host} : {err}")
+            _LOGGER.exception(f"X0020 - Error occurred: {self._host} : {err}")
             return None
 
     def _sendCommand(self, code):
@@ -422,15 +473,15 @@ class SkyQRemote:
             client = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         except socket.error as err:
             _LOGGER.exception(
-                f"X0040 - Failed to create socket when sending command: {self.host} : {err}"
+                f"X0040 - Failed to create socket when sending command: {self._host} : {err}"
             )
             return
 
         try:
-            client.connect((self.host, self._port))
+            client.connect((self._host, self._port))
         except Exception as err:
             _LOGGER.exception(
-                f"X0050 - Failed to connect to client when sending command: {self.host} : {err}"
+                f"X0050 - Failed to connect to client when sending command: {self._host} : {err}"
             )
             return
 
@@ -453,7 +504,7 @@ class SkyQRemote:
 
             if time.time() > timeout:
                 _LOGGER.error(
-                    "E0030 - Timeout error sending command: {self.host} : {0}".format(
+                    "E0030 - Timeout error sending command: {self._host} : {0}".format(
                         str(code)
                     )
                 )
@@ -474,7 +525,7 @@ class SkyQRemote:
 
     def _setupDevice(self):
         """Set the remote up."""
-        deviceInfo = self._getDeviceInformation()
+        deviceInfo = self.getDeviceInformation()
         if not deviceInfo:
             return
 
@@ -485,21 +536,20 @@ class SkyQRemote:
             url_index += 1
 
         SkyQCountry = self._importCountry(deviceInfo)
-        if "serialNumber" in deviceInfo:
-            self.serialNumber = deviceInfo["serialNumber"]
 
-        self._remoteCountry = SkyQCountry(self.host)
-        self.country = self._remoteCountry.country
+        self._remoteCountry = SkyQCountry(self._host)
 
+        # This is here because otherwise I can never validate code for a foreign device
         if not self._test_channel:
             self._channels = self._http_json(REST_CHANNEL_LIST)
         else:
             self._channels = TEST_CHANNEL_LIST
 
         self.deviceSetup = True
+
         return
 
-    def _getDeviceInformation(self):
+    def _retrieveDeviceInformation(self):
         try:
             resp = self._http_json(REST_PATH_DEVICEINFO)
             return resp
@@ -509,32 +559,34 @@ class SkyQRemote:
         ):
             return None
         except Exception as err:
-            _LOGGER.exception(f"X0080 - Error occurred: {self.host} : {err}")
+            _LOGGER.exception(f"X0080 - Error occurred: {self._host} : {err}")
+            return None
+
+    def _retrieveSystemInformation(self):
+        try:
+            resp = self._http_json(REST_PATH_SYSTEMINFO)
+            return resp
+        except (
+            requests.exceptions.ConnectTimeout,
+            requests.exceptions.ConnectionError,
+        ):
+            return None
+        except Exception as err:
+            _LOGGER.exception(f"X0090 - Error occurred: {self._host} : {err}")
             return None
 
     def _importCountry(self, deviceInfo):
-        alpha3 = None
-        if self._overrideCountry:
-            alpha3 = self._overrideCountry
-        elif "countryCode" in deviceInfo:
-            alpha3 = deviceInfo["countryCode"].upper()
-
-        if not alpha3:
-            _LOGGER.error(f"E0050 - No country identified: {self.host}")
-            return None
-
-        if alpha3 in KNOWN_COUNTRIES:
-            alpha3 = KNOWN_COUNTRIES[alpha3]
-
         try:
-            country = pycountry.countries.get(alpha_3=alpha3).alpha_2.casefold()
+            country = pycountry.countries.get(
+                alpha_3=deviceInfo.epgCountryCode
+            ).alpha_2.casefold()
             SkyQCountry = importlib.import_module(
                 "pyskyqremote.country.remote_" + country
             ).SkyQCountry
 
         except (AttributeError, ModuleNotFoundError) as err:
             _LOGGER.warning(
-                f"W0030 - Invalid country, defaulting to GBR : {self.host} : {alpha3} : {err}"
+                f"W0030 - Invalid country, defaulting to GBR : {self._host} : {deviceInfo.epgCountryCode} : {err}"
             )
 
             from pyskyqremote.country.remote_gb import SkyQCountry
